local MogIt,mog = ...;

local GetCursorPosition = GetCursorPosition;
local ipairs = ipairs;

mog.manip = CreateFrame("Frame",nil,mog.frame);
mog.manip:SetAllPoints(mog.frame.Inset);

mog.manip.model = CreateFrame("DressUpModel",nil,mog.manip);
mog.manip.model:SetSize(195,195);
mog.manip.model:SetPoint("TOP",mog.manip,"TOP",0,-8);
mog.manip.model:SetBackdrop({
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
	--edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, --edgeSize = 16, 
	insets = {left = 0, right = 0, top = 0, bottom = 0},
});
mog.manip.model:SetBackdropColor(71/255,95/255,121/255,1);
mog.manip.model:EnableMouse(true);
mog.manip.model:EnableMouseWheel(true);
mog.manip.model:SetModelScale(2);
mog.manip.model:SetScript("OnHide",function(self)
	self:SetPosition(0,0,0);
	self:GetScript("OnMouseUp")(self);
end);
mog.manip.model:SetScript("OnShow",function(self)
	self:SetPosition(mog.settings.posZ,mog.settings.posX,mog.settings.posY);
end);
mog.manip.model:SetScript("OnMouseDown",function(self,btn)
	if btn == "LeftButton" then
		mog.manip.update.isRotating = 1;
	elseif btn == "RightButton" then
		mog.manip.update.isPosing = 1;
	end
	mog.manip.update.prevx,mog.manip.update.prevy = GetCursorPosition();
	mog.manip.update:Show();
end);
mog.manip.model:SetScript("OnMouseUp",function(self,btn)
	mog.manip.update:Hide();
	mog.manip.update.isRotating = nil;
	mog.manip.update.isPosing = nil;
end);
mog.manip.model:SetScript("OnMouseWheel",function(self,v)
	mog.settings.posZ = mog.settings.posZ + ((v > 0 and 0.6) or -0.6);
	mog.manip.mass(nil,true);
end);
mog.manip.update = CreateFrame("Frame",mog.manip);
mog.manip.update:SetScript("OnUpdate",function(self)
	local currentx,currenty = GetCursorPosition();
	if self.isRotating then
		mog.settings.face = mog.settings.face + ((currentx-self.prevx)/50);
		mog.manip.mass(true);
	elseif self.isPosing then
		mog.settings.posX = mog.settings.posX + ((currentx-self.prevx)/50);
		mog.settings.posY = mog.settings.posY + ((currenty-self.prevy)/50);
		mog.manip.mass(nil,true);
	end
	self.prevx,self.prevy = currentx,currenty;
end);

function mog.manip.mass(rotate,pose)
	if rotate then
		mog.manip.model:SetFacing(mog.settings.face);
	end
	if pose and mog.manip.model:IsShown() then
		mog.manip.model:SetPosition(mog.settings.posZ,mog.settings.posX,mog.settings.posY);
	end
	
	pose = pose and mog.grid:IsShown();
	for k,v in ipairs(mog.models) do
		if rotate then
			v:SetFacing(mog.settings.face);
		end
		if pose and v:IsShown() then
			v:SetPosition(mog.settings.posZ,mog.settings.posX,mog.settings.posY);
		end
	end
end

mog.manip.helpZoom = mog.manip:CreateFontString(nil,"ARTWORK","GameFontGreenSmall");
mog.manip.helpZoom:SetPoint("BOTTOMLEFT",mog.manip,"BOTTOMLEFT",10,8);
mog.manip.helpZoom:SetText("Zoom");

mog.manip.keyZoom = mog.manip:CreateFontString(nil,"ARTWORK","GameFontHighlightSmall");
mog.manip.keyZoom:SetPoint("BOTTOMRIGHT",mog.manip,"BOTTOMRIGHT",-44,8);
mog.manip.keyZoom:SetText("Scroll wheel or");

mog.manip.zoomIn = CreateFrame("Button",nil,mog.manip);
mog.manip.zoomIn:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
mog.manip.zoomIn:SetPushedTexture("Interface\\Buttons\\UI-PlusButton-Down");
mog.manip.zoomIn:SetHighlightTexture("Interface\\Buttons\\UI-PlusButton-Hilight","ADD");
mog.manip.zoomIn:SetSize(16,16);
mog.manip.zoomIn:SetPoint("LEFT",mog.manip.keyZoom,"RIGHT",2,0);
mog.manip.zoomIn:SetScript("OnMouseDown",function(self)
	mog.manip.zoom.dir = true;
	mog.manip.zoom:Show();
end);
mog.manip.zoomIn:SetScript("OnMouseUp",function(self)
	mog.manip.zoom:Hide();
end);

mog.manip.zoomOut = CreateFrame("Button",nil,mog.manip);
mog.manip.zoomOut:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
mog.manip.zoomOut:SetPushedTexture("Interface\\Buttons\\UI-MinusButton-Down");
mog.manip.zoomOut:SetHighlightTexture("Interface\\Buttons\\UI-PlusButton-Hilight","ADD");
mog.manip.zoomOut:SetSize(16,16);
mog.manip.zoomOut:SetPoint("LEFT",mog.manip.zoomIn,"RIGHT",0,0);
mog.manip.zoomOut:SetScript("OnMouseDown",function(self)
	mog.manip.zoom.dir = false;
	mog.manip.zoom:Show();
end);
mog.manip.zoomOut:SetScript("OnMouseUp",function(self)
	mog.manip.zoom:Hide();
end);

mog.manip.helpMove = mog.manip:CreateFontString(nil,"ARTWORK","GameFontGreenSmall");
mog.manip.helpMove:SetPoint("BOTTOMLEFT",mog.manip.helpZoom,"TOPLEFT",0,5);
mog.manip.helpMove:SetText("Move");

mog.manip.keyMove = mog.manip:CreateFontString(nil,"ARTWORK","GameFontHighlightSmall");
mog.manip.keyMove:SetPoint("BOTTOMRIGHT",mog.manip.keyZoom,"TOPRIGHT",34,5);
mog.manip.keyMove:SetText("Right click and drag");

mog.manip.helpRotate = mog.manip:CreateFontString(nil,"ARTWORK","GameFontGreenSmall");
mog.manip.helpRotate:SetPoint("BOTTOMLEFT",mog.manip.helpMove,"TOPLEFT",0,5);
mog.manip.helpRotate:SetText("Rotate");

mog.manip.keyRotate = mog.manip:CreateFontString(nil,"ARTWORK","GameFontHighlightSmall");
mog.manip.keyRotate:SetPoint("BOTTOMRIGHT",mog.manip.keyMove,"TOPRIGHT",0,5);
mog.manip.keyRotate:SetText("Left click and drag");

mog.manip.reset = CreateFrame("Button",nil,mog.manip,"UIPanelButtonTemplate");
mog.manip.reset:SetText(RESET);
mog.manip.reset:SetSize(80,22);
mog.manip.reset:SetPoint("TOPLEFT",mog.frame.Inset,"BOTTOMLEFT");
mog.manip.reset:SetScript("OnClick",function(self,btn)
	mog.settings.posX = 0;
	mog.settings.posY = 0;
	mog.settings.posZ = 0;
	mog.settings.face = 0;
	mog.manip.mass(true,true);
end);

mog.manip.zoom = CreateFrame("Frame",nil,mog.manip);
mog.manip.zoom:Hide();
mog.manip.zoom:SetScript("OnUpdate",function(self,elapsed)
	if self.dir then
		mog.settings.posZ = mog.settings.posZ + elapsed*3;
	else
		mog.settings.posZ = mog.settings.posZ - elapsed*3;
	end
	mog.manip.mass(nil,true);
end);